@@ -24,19 +24,31 @@ import java.lang.reflect.ParameterizedType;
 import java.lang.reflect.Type;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collections;
+import java.util.HashSet;
 import java.util.List;
-
+import java.util.Set;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
 import javax.wsdl.Operation;
 import javax.xml.bind.annotation.XmlNsForm;
 import javax.xml.bind.annotation.XmlSchema;
+import javax.xml.bind.annotation.XmlSeeAlso;
 import javax.xml.namespace.QName;
+import javax.xml.ws.Action;
 import javax.xml.ws.AsyncHandler;
+import javax.xml.ws.BindingType;
+import javax.xml.ws.FaultAction;
 import javax.xml.ws.Service;
 import javax.xml.ws.Service.Mode;
 import javax.xml.ws.WebFault;
+import javax.xml.ws.WebServiceFeature;
+import javax.xml.ws.soap.Addressing;
+import javax.xml.ws.soap.AddressingFeature;
+import javax.xml.ws.soap.MTOM;
+import javax.xml.ws.soap.MTOMFeature;
+import javax.xml.ws.soap.SOAPBinding;
 
 import org.apache.cxf.binding.AbstractBindingFactory;
 import org.apache.cxf.common.classloader.ClassLoaderUtils;
@@ -46,21 +58,25 @@ import org.apache.cxf.common.util.StringUtils;
 import org.apache.cxf.databinding.source.SourceDataBinding;
 import org.apache.cxf.endpoint.Endpoint;
 import org.apache.cxf.endpoint.EndpointException;
-import org.apache.cxf.frontend.SimpleMethodDispatcher;
 import org.apache.cxf.helpers.CastUtils;
+import org.apache.cxf.jaxb.JAXBDataBinding;
 import org.apache.cxf.jaxws.JAXWSMethodDispatcher;
+import org.apache.cxf.jaxws.JAXWSProviderMethodDispatcher;
+import org.apache.cxf.jaxws.WrapperClassGenerator;
 import org.apache.cxf.jaxws.interceptors.DispatchInDatabindingInterceptor;
 import org.apache.cxf.jaxws.interceptors.DispatchOutDatabindingInterceptor;
 import org.apache.cxf.jaxws.interceptors.WebFaultOutInterceptor;
 import org.apache.cxf.service.factory.AbstractServiceConfiguration;
 import org.apache.cxf.service.factory.ReflectionServiceFactoryBean;
 import org.apache.cxf.service.factory.ServiceConstructionException;
-import org.apache.cxf.service.invoker.Invoker;
 import org.apache.cxf.service.model.BindingInfo;
 import org.apache.cxf.service.model.EndpointInfo;
+import org.apache.cxf.service.model.FaultInfo;
 import org.apache.cxf.service.model.InterfaceInfo;
+import org.apache.cxf.service.model.MessageInfo;
 import org.apache.cxf.service.model.OperationInfo;
 import org.apache.cxf.service.model.ServiceInfo;
+import org.apache.cxf.ws.addressing.JAXWSAConstants;
 import org.apache.cxf.wsdl11.WSDLServiceBuilder;
 
 /**
@@ -70,20 +86,27 @@ import org.apache.cxf.wsdl11.WSDLServiceBuilder;
  * @see org.apache.cxf.jaxws.JaxWsServerFactoryBean
  */
 public class JaxWsServiceFactoryBean extends ReflectionServiceFactoryBean {
+    // used to tag property on service.
+    public static final  String WS_FEATURES = "JAXWS-WS-FEATURES";
     private static final Logger LOG = LogUtils.getLogger(JaxWsServiceFactoryBean.class);
     
     private AbstractServiceConfiguration jaxWsConfiguration;
 
     private JaxWsImplementorInfo implInfo;
 
-    private JAXWSMethodDispatcher methodDispatcher;
+    private List<WebServiceFeature> wsFeatures;
 
+    private boolean wrapperBeanGenerated;
+    private Set<Class<?>> wrapperClasses;
+    
+    
     public JaxWsServiceFactoryBean() {
         getIgnoredClasses().add(Service.class.getName());
         
         //the JAXWS-RI doesn't qualify the schemas for the wrapper types
         //and thus won't work if we do.
         setQualifyWrapperSchema(false);
+        initSchemaLocations();
     }
 
     public JaxWsServiceFactoryBean(JaxWsImplementorInfo implInfo) {
@@ -91,6 +114,49 @@ public class JaxWsServiceFactoryBean extends ReflectionServiceFactoryBean {
         this.implInfo = implInfo;
         initConfiguration(implInfo);
         this.serviceClass = implInfo.getEndpointClass();
+        loadWSFeatureAnnotation(implInfo.getSEIClass(), implInfo.getImplementorClass());
+    }
+
+    private void initSchemaLocations() {
+        this.schemaLocationMapping.put(JAXWSAConstants.NS_WSA, 
+                                       JAXWSAConstants.WSA_XSD);
+    }
+    
+    private void loadWSFeatureAnnotation(Class<?> serviceClass, Class<?> implementorClass) {
+        List<WebServiceFeature> features = new ArrayList<WebServiceFeature>();
+        MTOM mtom = implInfo.getImplementorClass().getAnnotation(MTOM.class);        
+        if (mtom == null && serviceClass != null) {
+            mtom = serviceClass.getAnnotation(MTOM.class);
+        }
+        if (mtom != null) {
+            features.add(new MTOMFeature(mtom.enabled(), mtom.threshold()));
+        } else {
+            //deprecated way to set mtom
+            BindingType bt = implInfo.getImplementorClass().getAnnotation(BindingType.class);
+            if (bt != null
+                && (SOAPBinding.SOAP11HTTP_MTOM_BINDING.equals(bt.value())
+                || SOAPBinding.SOAP12HTTP_MTOM_BINDING.equals(bt.value()))) {
+                features.add(new MTOMFeature(true));                
+            }
+        }
+        
+
+        Addressing addressing = null;
+        if (implementorClass != null) {
+            addressing = implementorClass.getAnnotation(Addressing.class);
+        }
+
+        if (addressing == null && serviceClass != null) {
+            addressing = serviceClass.getAnnotation(Addressing.class);
+        }
+
+        if (addressing != null) {
+            features.add(new AddressingFeature(addressing.enabled(), addressing.required()));
+        }
+
+        if (features.size() > 0) {
+            wsFeatures = features;
+        }
     }
 
     @Override
@@ -102,24 +168,11 @@ public class JaxWsServiceFactoryBean extends ReflectionServiceFactoryBean {
         return s;
     }
 
-    @Override
-    protected Invoker createInvoker() {
-        return null;
-    }
-
-    protected SimpleMethodDispatcher getMethodDispatcher() {
-        return methodDispatcher;
-    }
 
     @Override
     public void setServiceClass(Class<?> serviceClass) {
-        JaxWsImplementorInfo info = new JaxWsImplementorInfo(serviceClass);
-        setJaxWsImplementorInfo(info);
-        super.setServiceClass(info.getEndpointClass());
-    }
-    @Override
-    protected void checkServiceClassAnnotations(Class<?> sc) {
-        //no need to check
+        setJaxWsImplementorInfo(new JaxWsImplementorInfo(serviceClass));
+        super.setServiceClass(getJaxWsImplementorInfo().getEndpointClass());
     }
 
     @Override
@@ -141,27 +194,18 @@ public class JaxWsServiceFactoryBean extends ReflectionServiceFactoryBean {
     }
 
     @Override
-    public Endpoint createEndpoint(EndpointInfo ei) throws EndpointException {
-        return new JaxWsEndpointImpl(getBus(), getService(), ei, implInfo);
+    public Endpoint createEndpoint(EndpointInfo ei) throws EndpointException {        
+        return new JaxWsEndpointImpl(getBus(), getService(), ei, implInfo, wsFeatures, 
+                                     this.getFeatures(), this.isFromWsdl());
     }
 
     @Override
     protected void initializeWSDLOperation(InterfaceInfo intf, OperationInfo o, Method method) {
         method = ((JaxWsServiceConfiguration)jaxWsConfiguration).getDeclaredMethod(method);
-
+        o.setProperty(Method.class.getName(), method);
+        o.setProperty(METHOD, method);
         initializeWrapping(o, method);
 
-        bindOperation(o, method);
-
-        // rpc out-message-part-info class mapping
-        Operation op = (Operation)o.getProperty(WSDLServiceBuilder.WSDL_OPERATION);
-
-        initializeClassInfo(o, method, op == null ? null
-            : CastUtils.cast(op.getParameterOrdering(), String.class));
-    }
-    
-    protected void bindOperation(OperationInfo op, Method method) {
-        
         try {
             // Find the Async method which returns a Response
             Method responseMethod = method.getDeclaringClass().getDeclaredMethod(method.getName() + "Async",
@@ -176,15 +220,20 @@ public class JaxWsServiceFactoryBean extends ReflectionServiceFactoryBean {
                 .getDeclaredMethod(method.getName() + "Async",
                                    asyncHandlerParams.toArray(new Class<?>[asyncHandlerParams.size()]));
 
-            getMethodDispatcher().bind(op, method, responseMethod, futureMethod);
+            getMethodDispatcher().bind(o, method, responseMethod, futureMethod);
 
         } catch (SecurityException e) {
             throw new ServiceConstructionException(e);
         } catch (NoSuchMethodException e) {
-            getMethodDispatcher().bind(op, method);
+            getMethodDispatcher().bind(o, method);
         }
-    }
 
+        // rpc out-message-part-info class mapping
+        Operation op = (Operation)o.getProperty(WSDLServiceBuilder.WSDL_OPERATION);
+
+        initializeClassInfo(o, method, op == null ? null
+            : CastUtils.cast(op.getParameterOrdering(), String.class));
+    }
 
     @Override
     protected void initializeWSDLOperations() {
@@ -348,6 +397,7 @@ public class JaxWsServiceFactoryBean extends ReflectionServiceFactoryBean {
             getServiceConfigurations().add(0, jaxWsConfiguration);
             setWrapped(false);
             setDataBinding(new SourceDataBinding());
+            setMethodDispatcher(new JAXWSProviderMethodDispatcher(implInfo));
         } else {
             jaxWsConfiguration = new JaxWsServiceConfiguration();
             jaxWsConfiguration.setServiceFactory(this);
@@ -360,8 +410,109 @@ public class JaxWsServiceFactoryBean extends ReflectionServiceFactoryBean {
                     setQualifyWrapperSchema(true);
                 }
             }
+            setMethodDispatcher(new JAXWSMethodDispatcher(implInfo));
+        }
+        loadWSFeatureAnnotation(ii.getSEIClass(), ii.getImplementorClass());        
+    }
+
+    public List<WebServiceFeature> getWsFeatures() {
+        return wsFeatures;
+    }
+
+    public void setWsFeatures(List<WebServiceFeature> wsFeatures) {
+        this.wsFeatures = wsFeatures;
+    }
+
+    private FaultInfo getFaultInfo(final OperationInfo operation, final Class expClass) {
+        for (FaultInfo fault : operation.getFaults()) {
+            if (fault.getProperty(Class.class.getName()) == expClass
+                || fault.getProperty(Class.class.getName()) == expClass) {
+                return fault;
+            }
+        }
+        return null;
+    }
+    private void buildWSAActions(OperationInfo operation, Method method) {
+        //nothing
+        if (method == null) {
+            return;
+        }
+
+        Action action = method.getAnnotation(Action.class);
+        if (action == null) {
+            return;
+        }
+        MessageInfo input = operation.getInput();
+        if (!StringUtils.isEmpty(action.input())) {
+            input.addExtensionAttribute(JAXWSAConstants.WSAW_ACTION_QNAME, action.input());
         }
-        methodDispatcher = new JAXWSMethodDispatcher(implInfo);
         
+        MessageInfo output = operation.getOutput();
+        if (output != null && !StringUtils.isEmpty(action.output())) {
+            output.addExtensionAttribute(JAXWSAConstants.WSAW_ACTION_QNAME, action.output());
+        }
+        
+        FaultAction[] faultActions = action.fault();
+        if (faultActions != null 
+            && faultActions.length > 0 
+            && operation.getFaults() != null) {
+            for (FaultAction faultAction : faultActions) {                
+                FaultInfo faultInfo = getFaultInfo(operation, faultAction.className());
+                faultInfo.addExtensionAttribute(JAXWSAConstants.WSAW_ACTION_QNAME, 
+                                                faultAction.value());
+                if (operation.isUnwrappedCapable()) {
+                    faultInfo = getFaultInfo(operation.getUnwrappedOperation(), faultAction.className());
+                    faultInfo.addExtensionAttribute(JAXWSAConstants.WSAW_ACTION_QNAME, 
+                                                    faultAction.value());
+                }
+            }
+        }        
+    }
+    
+    @Override
+    protected OperationInfo createOperation(ServiceInfo serviceInfo, InterfaceInfo intf, Method m) {
+        OperationInfo op = super.createOperation(serviceInfo, intf, m);
+        if (op.getUnwrappedOperation() != null) {
+            op = op.getUnwrappedOperation();
+        }
+        buildWSAActions(op, m);
+        return op;
+    }
+    
+    @Override
+    protected Set<Class<?>> getExtraClass() {
+        Set<Class<?>> classes = new HashSet<Class<?>>();
+        if (!wrapperBeanGenerated) {
+            wrapperClasses = generatedWrapperBeanClass();
+        }
+        if (wrapperClasses != null) {
+            classes.addAll(wrapperClasses);
+        }
+        
+        XmlSeeAlso xmlSeeAlsoAnno = getServiceClass().getAnnotation(XmlSeeAlso.class);
+        
+        if (xmlSeeAlsoAnno != null && xmlSeeAlsoAnno.value() != null) {
+            for (int i = 0; i < xmlSeeAlsoAnno.value().length; i++) {
+                classes.add(xmlSeeAlsoAnno.value()[i]);
+            }
+        }
+        return classes;
+    }
+    
+    private Set<Class<?>> generatedWrapperBeanClass() {
+        if (getDataBinding() instanceof JAXBDataBinding) {
+            ServiceInfo serviceInfo = getService().getServiceInfos().get(0);
+            WrapperClassGenerator wrapperGen = new WrapperClassGenerator(this,
+                                                                         serviceInfo.getInterface(),
+                                                                         getQualifyWrapperSchema());
+            return wrapperGen.generate();            
+        }
+        return Collections.emptySet();
+    }
+
+    @Override
+    protected void buildServiceFromClass() {
+        super.buildServiceFromClass();
+        getService().put(WS_FEATURES, getWsFeatures()); 
     }
 }
